import 'package:analyzer/dart/element/element.dart';
import 'package:like_sealed_gen/src/util.dart';

class SealedCollectionContent {
  final String className;
  final List<ClassElement> types;
  final bool isShortName;
  final List<ImportElement> importElement;

  SealedCollectionContent(
      this.isShortName, this.className, this.types, this.importElement);

  String content() {
    return """
   class ${className}s {
   ${_methods()}
   }
   """;
  }

  String _methods() {
    final methods = <String>[];
    for (var type in types) {
      if (type.isAbstract) {
        continue;
      }
      for (var constructor in type.constructors) {
        if (constructor.isAbstract) {
          continue;
        }
        String functionName = type.name;
        if (isShortName) {
          functionName = functionName.removeIf(this.className);
        }
        functionName = functionName.firstToLower;
        var className = type.name;
        var returnType =
            constructor.returnType.getDisplayString(withNullability: true);

        if (constructor.name.isNotEmpty) {
          functionName = functionName + constructor.name.firstToUpper;
          className = className + "." + constructor.name;
        }

        final textConstructor = constructor.parameters.map(
          (parameter) {
            if (parameter.isNamed) {
              return parameter.name + " : " + parameter.name;
            } else {
              return parameter.name;
            }
          },
        ).join(" , ");
        var parameters = "";
        var hasOptional = false;
        var hasNamed = false;
        for (var parameter in constructor.parameters) {
          if (!hasOptional && parameter.isOptional) {
            hasOptional = true;
            if (parameter.isNamed) {
              hasNamed = true;
              parameters += "{";
            } else {
              parameters += "[";
            }
          }
          final prefix = getImportPrefix(parameter);
          if (prefix != null) {
            parameters += "$prefix.";
          }
          parameters += parameter.type.getDisplayString(withNullability: true);
          parameters += " ";
          parameters += parameter.name;
          if (parameter.hasDefaultValue) {
            parameters += "= ${parameter.defaultValueCode}";
          }
          parameters += ", ";
        }
        if (hasOptional) {
          if (hasNamed) {
            parameters += "}";
          } else {
            parameters += "]";
          }
        }
        final function = """
        static $returnType $functionName($parameters){
          return $className($textConstructor);
        } 
        """;

        methods.add(function);
      }
    }
    return methods.join("\n");
  }

  String? getImportPrefix(
    ParameterElement parameter,
  ) {
    final importIdentifier = parameter.type.element?.library?.identifier;
    if (importIdentifier == null) {
      return null;
    }
    final import = parameter.library!.imports.firstWhereOrNull(
      (element) => element.importedLibrary!.identifier == importIdentifier,
    );
    if (import?.prefix != null) {
      return import!.prefix!.name;
    }
    return null;
  }
}
