import 'package:analyzer/dart/element/element.dart';
import 'package:like_sealed_gen/src/util.dart';

class SealedSwitchImplContent {
  final String className;
  final List<ClassElement> types;
  final bool isShortName;

  SealedSwitchImplContent(this.isShortName, this.className, this.types);

  String content() {
    String typeChecker = "";
    String field = "";
    String constructorParameter = "";
    List<String> requiredConstructorParameter = [];
    List<String> requiredConstructorSuper = [];
    String caseFunction = "";

    for (var type in types) {
      final typeName = type.name;
      String functionName = typeName;
      if (isShortName) {
        functionName = functionName.removeIf(className);
      }
      final filedName = functionName.isEmpty
          ? className.firstToLower
          : functionName.firstToLower;

      requiredConstructorParameter
          .add("final T Function($typeName)? ${filedName}");

      requiredConstructorSuper.add("_${filedName}=${filedName}");
      constructorParameter += "this._${filedName},";
      field += "final T Function($typeName)?  _${filedName};";

      if (typeName == className) {
        typeChecker +=
            "(type is $typeName&& _${filedName} != null)? onDefault(type) : throw UnimplementedError()";

        caseFunction += "T onDefault($typeName value)=> _${filedName}!(value);";
      } else {
        typeChecker +=
            "(type is $typeName && _${filedName} != null) ? on$functionName(type) : ";

        caseFunction +=
            "T on$functionName($typeName value)=> _${filedName}!(value) ;";
      }
    }

    return """
    class ${className}SwitchImpl<T> implements ${className}Switch<T> {
      ${field}
      ${className}SwitchImpl({${requiredConstructorParameter.join(",")}}): ${requiredConstructorSuper.join(",")};
      
      @Deprecated("use base constructor or required")
      ${className}SwitchImpl.fill($constructorParameter);
      
       ${className}SwitchImpl.required({${requiredConstructorParameter.map((e) => "required $e").join(",")}}): ${requiredConstructorSuper.join(",")};
     
      T switchCase($className type)=>$typeChecker;
      
      $caseFunction
    }
    """;
  }
}
