import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:like_sealed/like_sealed.dart';
import 'package:like_sealed_gen/src/content/sealed_content.dart';
import 'package:source_gen/source_gen.dart';

class SealedGenerator extends GeneratorForAnnotation<LikeSealed> {
  @override
  generateForAnnotatedElement(
    Element element,
    ConstantReader annotation,
    _,
  ) {
    if (element is ClassElement) {
      final isShortName =
          annotation.objectValue.getField("shortName")!.toBoolValue()!;
      final isCollection =
          annotation.objectValue.getField("collection")!.toBoolValue()!;
      final isSwitchInterface =
          annotation.objectValue.getField("switchInterface")!.toBoolValue()!;
      final isSwitchImpl =
          annotation.objectValue.getField("switchImpl")!.toBoolValue()!;

      final className = element.name;

      final extendedClasses = getSubtypes(element.library, className);

      extendedClasses.add(element);

      return ContentFormat(
        isShortName,
        isCollection,
        isSwitchInterface,
        isSwitchImpl,
      ).build(
        element.library.imports,
        className,
        extendedClasses,
      );
    }
    return "";
  }

  List<ClassElement> getSubtypes(LibraryElement lib, String type) {
    final types = <ClassElement>[];
    final libraryReader = LibraryReader(lib);

    libraryReader.classes.forEach((classElement) {
      if (isExtended(classElement.supertype, type)) {
        types.add(classElement);
      }
    });

    return types;
  }

  bool isExtended(InterfaceType? element, String type) {
    if (element == null) {
      return false;
    } else if (element.getDisplayString(withNullability: false) == type) {
      return true;
    } else {
      return isExtended(element.superclass, type);
    }
  }
}
